/**
 * Evaluator for pudding.
 * @constructor PuddingEvaluator
 * @param {object} config
 */

'use strict'

const extend = require('extend')
const parseExpression = require('./parsing/parse_expression')
const objnest = require('objnest')

/** @lends PuddingEvaluator */
function PuddingEvaluator (config) {
  const s = this
  extend(s, config)
}

PuddingEvaluator.prototype = {
  /**
   * Eval value.
   * @param {*} src - Source to eval.
   * @param {object} context - Evaluation context.
   * @returns {*} - Evaluated value.
   */
  eval(src, context) {
    let s = this
    if (Array.isArray(src)) {
      return s.evalArray(src, context)
    }
    switch (typeof src) {
      case 'string':
        return s.evalString(src, context)
      case 'function':
        return s.evalFunction(src, context)
      case 'object':
        return s.evalObject(src, context)
      default:
        return src
    }
  },
  /**
   * Eval array value.
   * @param {Array} src - Source to eval.
   * @param {object} context - Evaluation context.
   * @returns {Array} - Evaluated value.
   */
  evalArray (src, context) {
    const s = this
    return src.map((src) => {
      return s.eval(src, context)
    })
  },
  /**
   * Eval function value.
   * @param {function} src - Source to eval.
   * @param {object} context - Evaluation context.
   * @returns {*} - Evaluated value.
   */
  evalFunction (src, context) {
    const s = this
    return s.eval(src(context), context)
  },
  /**
   * Eval object value.
   * @param {object} src - Source to eval.
   * @param {object} context - Evaluation context.
   * @returns {*} - Evaluated value.
   */
  evalObject (src, context) {
    const s = this
    let data = objnest.flatten(src)
    Object.keys(data).forEach((key) => {
      data[ key ] = s.eval(data[ key ], context)
    })
    return objnest.expand(data)
  },
  /**
   * Eval string value.
   * @param {object} src - Source to eval.
   * @param {object} context - Evaluation context.
   * @returns {*} - Evaluated value.
   */
  evalString (src, context) {
    const s = this
    let data = String(src)
    let expressionPattern = [].concat(s.expressionPattern || [])
    for (let i = 0, len = expressionPattern.length; i < len; i++) {
      data = parseExpression(data, context, { pattern: expressionPattern[ i ] })
    }
    return data
  },
  expressionPattern: [
    /#{(.*?)}/g,
    /\${(.*?)}/g,
    /{{(.*?)}}/g
  ]
}

module.exports = PuddingEvaluator
