/**
 * Evaluator for pudding.
 * @constructor PuddingEvaluator
 * @param {object} config
 */

'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var extend = require('extend');
var parseExpression = require('./parsing/parse_expression');
var objnest = require('objnest');

/** @lends PuddingEvaluator */
function PuddingEvaluator(config) {
  var s = this;
  extend(s, config);
}

PuddingEvaluator.prototype = {
  /**
   * Eval value.
   * @param {*} src - Source to eval.
   * @param {object} context - Evaluation context.
   * @returns {*} - Evaluated value.
   */
  eval: function _eval(src, context) {
    var s = this;
    if (Array.isArray(src)) {
      return s.evalArray(src, context);
    }
    switch (typeof src === 'undefined' ? 'undefined' : _typeof(src)) {
      case 'string':
        return s.evalString(src, context);
      case 'function':
        return s.evalFunction(src, context);
      case 'object':
        return s.evalObject(src, context);
      default:
        return src;
    }
  },

  /**
   * Eval array value.
   * @param {Array} src - Source to eval.
   * @param {object} context - Evaluation context.
   * @returns {Array} - Evaluated value.
   */
  evalArray: function evalArray(src, context) {
    var s = this;
    return src.map(function (src) {
      return s.eval(src, context);
    });
  },

  /**
   * Eval function value.
   * @param {function} src - Source to eval.
   * @param {object} context - Evaluation context.
   * @returns {*} - Evaluated value.
   */
  evalFunction: function evalFunction(src, context) {
    var s = this;
    return s.eval(src(context), context);
  },

  /**
   * Eval object value.
   * @param {object} src - Source to eval.
   * @param {object} context - Evaluation context.
   * @returns {*} - Evaluated value.
   */
  evalObject: function evalObject(src, context) {
    var s = this;
    var data = objnest.flatten(src);
    Object.keys(data).forEach(function (key) {
      data[key] = s.eval(data[key], context);
    });
    return objnest.expand(data);
  },

  /**
   * Eval string value.
   * @param {object} src - Source to eval.
   * @param {object} context - Evaluation context.
   * @returns {*} - Evaluated value.
   */
  evalString: function evalString(src, context) {
    var s = this;
    var data = String(src);
    var expressionPattern = [].concat(s.expressionPattern || []);
    for (var i = 0, len = expressionPattern.length; i < len; i++) {
      data = parseExpression(data, context, { pattern: expressionPattern[i] });
    }
    return data;
  },

  expressionPattern: [/#{(.*?)}/g, /\${(.*?)}/g, /{{(.*?)}}/g]
};

module.exports = PuddingEvaluator;
//# sourceMappingURL=data:application/json;base64,bnVsbA==